﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.IO;

namespace CodeSnippetEditor
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private readonly string _snippetsFolder = System.IO.Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), @"Visual Studio 2010\Code Snippets\Visual C#\My Code Snippets");
        private const string CUSTOM_CODE_TITLE = "CUSTOM_CODE_TITLE";
        private const string CUSTOM_CODE_AUTHOR = "CUSTOM_CODE_AUTHOR";
        private const string CUSTOM_CODE_DESCRIPTION = "CUSTOM_CODE_DESCRIPTION";
        private const string CUSTOM_CODE_SNIPPET = "CUSTOM_CODE_SNIPPET";

        public static readonly RoutedCommand OKCommand = new RoutedCommand("OKCommand", typeof(MainWindow));
        public static readonly RoutedCommand CancelCommand = new RoutedCommand("CancelCommand", typeof(MainWindow));

        public MainWindow()
        {
            InitializeComponent();
            CommandBindings.Add(new CommandBinding(OKCommand, new ExecutedRoutedEventHandler(OKCommand_Executed)));
            CommandBindings.Add(new CommandBinding(CancelCommand, new ExecutedRoutedEventHandler(CancelCommand_Executed)));
            this.Loaded += (s, e) => { txtTitle.Focus(); };
        }

        private void OKCommand_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            if (!Directory.Exists(_snippetsFolder))
                Directory.CreateDirectory(_snippetsFolder);

            if (string.IsNullOrWhiteSpace(txtTitle.Text))
            {
                MessageBox.Show("Title cannot be empty.");
                return;
            }
            if (string.IsNullOrWhiteSpace(txtCodeSnippet.Text))
            {
                MessageBox.Show("Code Snippet cannot be empty.");
                return;
            }

            string fileName = txtTitle.Text.Trim();

            if (Directory.EnumerateFiles(_snippetsFolder, "*.snippet", SearchOption.TopDirectoryOnly).
                FirstOrDefault(p => string.Equals(System.IO.Path.GetFileNameWithoutExtension(p), fileName, StringComparison.OrdinalIgnoreCase)) != null)
            {
                var dialogResult = MessageBox.Show(this, "This name already exists, do you want to override it? OK to override or Cancel to specify a new name.", "", MessageBoxButton.OKCancel, MessageBoxImage.Warning);
                if (dialogResult == MessageBoxResult.Cancel)
                    return;
            }

            try
            {
                var streamInfo = Application.GetResourceStream(new Uri("pack://application:,,,/Template.snippet"));
                using (StreamReader sr = new StreamReader(streamInfo.Stream))
                {
                    string result = sr.ReadToEnd();
                    result = result.Replace(CUSTOM_CODE_TITLE, fileName).Replace(CUSTOM_CODE_AUTHOR, txtAuthor.Text).Replace(CUSTOM_CODE_DESCRIPTION, txtDesc.Text).Replace(CUSTOM_CODE_SNIPPET, txtCodeSnippet.Text);
                    File.WriteAllText(_snippetsFolder + "\\" + fileName + ".snippet", result);
                }
                MessageBox.Show("Succeeded.");
            }
            catch (Exception ex)
            {
                MessageBox.Show("Failed: " + ex.Message);
            }
        }

        private void CancelCommand_Executed(object sender, ExecutedRoutedEventArgs e)
        {
            this.Close();
        }

        protected override void OnPreviewKeyDown(KeyEventArgs e)
        {
            if (e.Key == Key.Escape)
                this.Close();
            base.OnPreviewKeyDown(e);
        }
    }
}
